package jumpstart.max.business.commons.exception;

import java.io.Serializable;

import javax.ejb.ApplicationException;

import jumpstart.max.business.domain.base.BaseEntity;
import jumpstart.max.util.ClassUtil;
import jumpstart.max.util.MessageUtil;

@ApplicationException(rollback = true)
@SuppressWarnings("serial")
public class OptimisticLockException extends ServiceException {
	private String _entityLabelMessageId;
	private Serializable _id;
	private BaseEntity _entity;
	private Exception _exception;

	public OptimisticLockException(BaseEntity entity, Serializable id, Exception exception) {
	
		// Don't convert the message ids to messages yet because we're in the server's locale, not the user's.

		super("OptimisticLockException");
		_entityLabelMessageId = ClassUtil.extractUnqualifiedName(entity);
		_id = id;
		_entity = entity;
		_exception = exception;
	}

	@Override
	public String getMessage() {
	
		// We deferred converting the message ids to messages until now, when we are more likely to be in the user's locale.
	
		Object[] msgArgs = new Object[] { 
			MessageUtil.toText(_entityLabelMessageId), 
			_id,
			_exception.getLocalizedMessage()
		};
		
		String msg = MessageUtil.toText(_messageId, msgArgs);
		return msg;
	}

	public BaseEntity getEntity() {
		return _entity;
	}

	public String getEntityLabelMessageId() {
		return _entityLabelMessageId;
	}

	public Exception getException() {
		return _exception;
	}

	public Serializable getId() {
		return _id;
	}
}
