package jumpstart.max.business.domain.security.iface;

import jumpstart.max.business.commons.exception.ServiceException;
import jumpstart.max.business.domain.security.Role;
import jumpstart.max.business.domain.security.User;
import jumpstart.max.business.domain.security.UserRole;

/**
 * The <code>ISecurityManagerSvcRemote</code> bean exposes the business methods in the interface.
 *
 * @author  Geoff Callender  JumpStart
 * @version $Revision: 1.29 $, $Date: 2005/08/21 13:04:12 $
 */
public interface ISecurityManagerSvcRemote {

	/***************************************************************************
	 * BUSINESS METHODS
	 **************************************************************************/

	/**
	 * This method provides a way for users to change their own password.
	 */
	void changeUserPassword(Long id, String currentPassword, String newPassword) throws ServiceException;
	
	/**
	 * This method provides a way for security officers to "reset" the password.
	 */
	void changeUserPassword(Long id, String newPassword) throws ServiceException;

	/***************************************************************************
	 * PERSISTENCE METHODS
	 **************************************************************************/

	// Role

	Role createRole(Role role) throws ServiceException;

	void changeRole(Role role) throws ServiceException;
	
	void deleteRole(Role role) throws ServiceException;

	// User

	User createUser(User user, String password) throws ServiceException;

	void changeUser(User user) throws ServiceException;
	
	void deleteUser(User user) throws ServiceException;

	// UserRole

	/**
	 * Use this "add" method when you don't care whether the parent(s) have changed while you've been preparing this child.
	 */
	UserRole addUserRole(UserRole userRole, Long userId, Long roleId) throws ServiceException;

	/**
	 * Use this "add" method when you want to ensure the parent(s) haven't changed while you've been preparing this child.
	 */
	UserRole addUserRole(UserRole userRole, User user, Role role) throws ServiceException;

	void changeUserRole(UserRole userRole) throws ServiceException;
	
	void removeUserRole(UserRole userRole) throws ServiceException;
}
