package jumpstart.max.web.commons.validator;

import org.apache.tapestry.form.IFormComponent;
import org.apache.tapestry.form.ValidationMessages;
import org.apache.tapestry.form.validator.BaseValidator;
import org.apache.tapestry.valid.ValidationConstraint;
import org.apache.tapestry.valid.ValidationStrings;
import org.apache.tapestry.valid.ValidatorException;
import org.joda.time.DateMidnight;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

public class MaxDateMidnight extends BaseValidator {
	private static final String isoFormatStr = "yyyy-MM-dd";
	private static final DateTimeFormatter isoFormatter = DateTimeFormat.forPattern(isoFormatStr);

	private DateMidnight _maxDateMidnight;

	public MaxDateMidnight() {
	}

	public MaxDateMidnight(String initializer) {
		super(initializer);
	}

	public void setMaxDateMidnight(String maxDateMidnight) {
		try {
			DateTime dt = isoFormatter.parseDateTime(maxDateMidnight);
			_maxDateMidnight = new DateMidnight(dt);
		}
		catch (IllegalArgumentException e) {
			throw new IllegalArgumentException("Value received for maxDateMidnight is \"" + maxDateMidnight
					+ "\" which does not match format \"" + isoFormatStr + "\".", e);
		}
	}

	public void validate(IFormComponent field, ValidationMessages messages, Object object) throws ValidatorException {
		DateMidnight dm = (DateMidnight) object;

		if (dm.compareTo(_maxDateMidnight) > 0)
			throw new ValidatorException(buildMessage(messages, field), ValidationConstraint.TOO_LARGE);

	}

	private String buildMessage(ValidationMessages messages, IFormComponent field) {
		return messages.formatValidationMessage(getMessage(), ValidationStrings.DATE_TOO_LATE, new Object[] {
				field.getDisplayName(), DateTimeFormat.forStyle("S-").print(_maxDateMidnight) });
	}

}
