package jumpstart.max.web.commons.validator;

import org.apache.tapestry.form.IFormComponent;
import org.apache.tapestry.form.ValidationMessages;
import org.apache.tapestry.form.validator.MinDate;
import org.apache.tapestry.valid.ValidationConstraint;
import org.apache.tapestry.valid.ValidationStrings;
import org.apache.tapestry.valid.ValidatorException;
import org.joda.time.DateMidnight;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

public class MinDateMidnight extends MinDate {
	private static final String isoFormatStr = "yyyy-MM-dd";
	private static final DateTimeFormatter isoFormatter = DateTimeFormat.forPattern(isoFormatStr);

	private DateMidnight _minDateMidnight;

	public MinDateMidnight() {
	}

	public MinDateMidnight(String initializer) {
		super(initializer);
	}

	public void setMinDateMidnight(String minDateMidnight) {
		try {
			DateTime dt = isoFormatter.parseDateTime(minDateMidnight);
			_minDateMidnight = new DateMidnight(dt);
		}
		catch (IllegalArgumentException e) {
			throw new IllegalArgumentException("Value received for minDateMidnight is \"" + minDateMidnight
					+ "\" which does not match format \"" + isoFormatStr + "\".", e);
		}
	}

	public void validate(IFormComponent field, ValidationMessages messages, Object object) throws ValidatorException {
		DateMidnight dm = (DateMidnight) object;

		if (dm.compareTo(_minDateMidnight) < 0)
			throw new ValidatorException(buildMessage(messages, field), ValidationConstraint.TOO_SMALL);

	}

	private String buildMessage(ValidationMessages messages, IFormComponent field) {
		return messages.formatValidationMessage(getMessage(), ValidationStrings.DATE_TOO_EARLY, new Object[] {
				field.getDisplayName(), DateTimeFormat.forStyle("S-").print(_minDateMidnight) });
	}

}
