// Based on com.nordija.tapestry.bayeux.* at http://tapestry-bayeux.sourceforge.net/
// Upgraded from Tapestry 3 to Tapestry 4.0 and added SubmitOnce and ImageSubmitOnce.

package jumpstart.max.web.components.bayeux.link;

import java.util.HashMap;
import java.util.Map;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.PageRenderSupport;
import org.apache.tapestry.TapestryUtils;

/**
 * A base component used to render a LinkSubmit, DirectLink or ActionLink with a
 * javascript onClick event handler that secures that the rendered component
 * cannot be activated more than once. <p/> The component takes care of any user
 * defined onClick eventhandler and secures that it is executed upon a user
 * click. <p/> <b>Note</b> The component uses javascript statehandling to
 * prevent more than one click, so be careful if you uses this component
 * together with the <i>AJAX</i> technology.
 * 
 * @author Jacob von Eyben www.nordija.com
 * @version $Id: ClickOnce.java,v 1.1 2005/09/11 17:50:43 jeyben Exp $
 */
public abstract class ClickOnce extends BaseComponent {

	public static final String DEFAULT_FUNCTION_NAME = "singleSubmit";

	public static final String SUBMITFLAG = "ongoingSubmit";

	public abstract String getOnclick();

	public abstract boolean getDisabled();

	public abstract void setFunctionName(String name);

	public abstract String getFunctionName();

	public abstract void setNestedFunctionCall(String nestedFunctionCall);

	public abstract String getNestedFunctionCall();

	public abstract void setIdParameter(String value);

	public abstract String getIdParameter();

	protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle) {
		if (!cycle.isRewinding()) {
			PageRenderSupport pageRenderSupport = TapestryUtils.getOptionalPageRenderSupport(cycle);
			if (pageRenderSupport == null) {
				throw new ApplicationRuntimeException("SingleSubmit component[" + getId()
						+ "] must be contained inside a body component", this, null, null);
			}
			if (getOnclick() == null || "".equals(getOnclick())) {
				setFunctionName(DEFAULT_FUNCTION_NAME);
			}
			else {
				setFunctionName(pageRenderSupport.getUniqueString(DEFAULT_FUNCTION_NAME + "Nested"));
			}
			setNestedFunctionCall(extractOnclick(getOnclick()));
		}
		else {
			// when rewinding just use some string to prevent nullpointer
			// exception in
			setFunctionName(DEFAULT_FUNCTION_NAME);
		}
		super.renderComponent(writer, cycle);
	}

	public Map getScriptSymbols() {
		Map<String, Object> ret = new HashMap<String, Object>();
		ret.put("disabled", Boolean.valueOf(getDisabled()));
		ret.put("submitflagName", SUBMITFLAG);
		ret.put("functionName", getFunctionName());
		ret.put("nestedFunctionCall", "".equals(getNestedFunctionCall()) ? null : getNestedFunctionCall());
		return ret;
	}

	/**
	 * Finds the onClick function call defined by the user. If the function
	 * begins with 'return' this is removed
	 * 
	 * @param onclick
	 * @return if onClick is <code>null</code> or empty, <code>null</code>
	 *         is returned, otherwise a string
	 */
	protected static String extractOnclick(String onclick) {
		String result = null;
		if (onclick != null && !("".equals(onclick))) {
			result = onclick.trim();
			// remove eventual "return " in beginning of the onClick.
			// done because we need to pick the returnvalue up in a variable in
			// our javascript.
			if (result.startsWith("return")) {
				result = result.replaceFirst("return", "");
			}
			result = result.trim();
		}
		return result;
	}

}
