package jumpstart.max.web.pages.login;

import jumpstart.max.business.commons.exception.ServiceException;
import jumpstart.max.business.domain.security.User;
import jumpstart.max.business.domain.security.iface.ISecurityFinderSvcLocal;
import jumpstart.max.web.commons.annotations.InjectPageLink;
import jumpstart.max.web.pages.base.SimpleBasePage;
import jumpstart.max.web.pages.starters.AlternatingTableComponentPage;
import jumpstart.max.web.pages.starters.AlternatingTablePage;
import jumpstart.max.web.pages.starters.CSSPage;
import jumpstart.max.web.pages.starters.ClientValidatorsWithFieldLabelsPage;
import jumpstart.max.web.pages.starters.DisablingValidatorsPage;
import jumpstart.max.web.pages.starters.DoItOnceAndRedirectPage;
import jumpstart.max.web.pages.starters.DoItOnceComponentsPage;
import jumpstart.max.web.pages.starters.DuplicateSubmissionPage;
import jumpstart.max.web.pages.starters.FriendlyURLsPage;
import jumpstart.max.web.pages.starters.MixedValidationPage;
import jumpstart.max.web.pages.starters.MixedValidationWithFieldLabelsPage;
import jumpstart.max.web.pages.starters.PreviewableTableComponentPage;
import jumpstart.max.web.pages.starters.PreviewableTablePage;
import jumpstart.max.web.pages.starters.RestartAndOtherEngineServicesPage;
import jumpstart.max.web.pages.starters.RestoreFromSessionPage;
import jumpstart.max.web.pages.starters.ReturnAnywhereUCPPage;
import jumpstart.max.web.pages.starters.ReturnAnywhereUHFPage;
import jumpstart.max.web.pages.starters.SimpleInputPage;
import jumpstart.max.web.pages.starters.SimpleOutputUsingActivatePage;
import jumpstart.max.web.pages.starters.SimpleRestoreOnReturnPage;
import jumpstart.max.web.pages.starters.SwitchableStylePage;
import jumpstart.max.web.pages.starters.TableComponentPage;
import jumpstart.max.web.pages.starters.TableComponentWithLinksPage;
import jumpstart.max.web.pages.starters.TablePage;
import jumpstart.max.web.pages.starters.TableViewComponentPage;
import jumpstart.max.web.pages.starters.TableWithLinksPage;
import jumpstart.max.web.pages.starters.TheSynchronizerTokenPage;
import jumpstart.max.web.pages.starters.ValidationPage;
import jumpstart.max.web.pages.starters.ValidatorsPage;
import jumpstart.max.web.pages.starters.ValidatorsWithFieldLabelsPage;
import jumpstart.max.web.pages.starters.WebFlow2StartPage;
import jumpstart.max.web.pages.starters.WebFlowStartPage;
import jumpstart.max.web.state.Visit;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tapestry.IAsset;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.annotations.Asset;
import org.apache.tapestry.engine.ILink;

/**
 * Allows the user to login, by providing userName and password. After
 * succesfully logging in.
 * 
 * @author Geoff Callender
 */

public abstract class LoginPage extends SimpleBasePage {
	static public final Log LOG = LogFactory.getLog(LoginPage.class);

	static public final String PAGE_NAME = "pages/login/LoginPage";

	@InjectPageLink(WelcomePage.PAGE_NAME)
	public abstract ILink getWelcomePageLink();

	@Asset("context:/css/login.css")
	public abstract IAsset getStylesheet();

	public abstract void setUserName(String value);

	public abstract String getLoginId();

	public abstract void setPassword(String value);
	public abstract String getPassword();

	public void activate() {
		// No preparation required before activation.
		getRequestCycle().activate(this);
	}

	public void doToSimpleOutputUsingActivate() {
		SimpleOutputUsingActivatePage nextPage = (SimpleOutputUsingActivatePage) getRequestCycle().getPage(
				SimpleOutputUsingActivatePage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToSimpleInput() {
		SimpleInputPage nextPage = (SimpleInputPage) getRequestCycle().getPage(SimpleInputPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToValidation() {
		ValidationPage nextPage = (ValidationPage) getRequestCycle().getPage(ValidationPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToValidators() {
		ValidatorsPage nextPage = (ValidatorsPage) getRequestCycle().getPage(ValidatorsPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToValidatorsWithFieldLabels() {
		ValidatorsWithFieldLabelsPage nextPage = (ValidatorsWithFieldLabelsPage) getRequestCycle().getPage(
				ValidatorsWithFieldLabelsPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToClientValidatorsWithFieldLabels() {
		ClientValidatorsWithFieldLabelsPage nextPage = (ClientValidatorsWithFieldLabelsPage) getRequestCycle().getPage(
				ClientValidatorsWithFieldLabelsPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToMixedValidation() {
		MixedValidationPage nextPage = (MixedValidationPage) getRequestCycle().getPage(MixedValidationPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToMixedValidationWithFieldLabels() {
		MixedValidationWithFieldLabelsPage nextPage = (MixedValidationWithFieldLabelsPage) getRequestCycle().getPage(
				MixedValidationWithFieldLabelsPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToRestartAndOtherEngineServices() {
		RestartAndOtherEngineServicesPage nextPage = (RestartAndOtherEngineServicesPage) getRequestCycle().getPage(
				RestartAndOtherEngineServicesPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToFriendlyURLs() {
		FriendlyURLsPage nextPage = (FriendlyURLsPage) getRequestCycle().getPage(FriendlyURLsPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToSimpleRestoreOnReturn() {
		SimpleRestoreOnReturnPage nextPage = (SimpleRestoreOnReturnPage) getRequestCycle().getPage(
				SimpleRestoreOnReturnPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToRestoreFromSession() {
		RestoreFromSessionPage nextPage = (RestoreFromSessionPage) getRequestCycle().getPage(
				RestoreFromSessionPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToReturnAnywhereUHF() {
		ReturnAnywhereUHFPage nextPage = (ReturnAnywhereUHFPage) getRequestCycle().getPage(
				ReturnAnywhereUHFPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToReturnAnywhereUCP() {
		ReturnAnywhereUCPPage nextPage = (ReturnAnywhereUCPPage) getRequestCycle().getPage(
				ReturnAnywhereUCPPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToCSS() {
		CSSPage nextPage = (CSSPage) getRequestCycle().getPage(CSSPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToSwitchableStyle() {
		SwitchableStylePage nextPage = (SwitchableStylePage) getRequestCycle().getPage(SwitchableStylePage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToTable() {
		TablePage nextPage = (TablePage) getRequestCycle().getPage(TablePage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToTableComponent() {
		TableComponentPage nextPage = (TableComponentPage) getRequestCycle().getPage(TableComponentPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToTableWithLinks() {
		TableWithLinksPage nextPage = (TableWithLinksPage) getRequestCycle().getPage(TableWithLinksPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToTableComponentWithLinks() {
		TableComponentWithLinksPage nextPage = (TableComponentWithLinksPage) getRequestCycle().getPage(
				TableComponentWithLinksPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToPreviewableTable() {
		PreviewableTablePage nextPage = (PreviewableTablePage) getRequestCycle()
				.getPage(PreviewableTablePage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToPreviewableTableComponent() {
		PreviewableTableComponentPage nextPage = (PreviewableTableComponentPage) getRequestCycle().getPage(
				PreviewableTableComponentPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToAlternatingTable() {
		AlternatingTablePage nextPage = (AlternatingTablePage) getRequestCycle()
				.getPage(AlternatingTablePage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToAlternatingTableComponent() {
		AlternatingTableComponentPage nextPage = (AlternatingTableComponentPage) getRequestCycle().getPage(
				AlternatingTableComponentPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToTableViewComponent() {
		TableViewComponentPage nextPage = (TableViewComponentPage) getRequestCycle().getPage(
				TableViewComponentPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToDuplicateSubmission() {
		DuplicateSubmissionPage nextPage = (DuplicateSubmissionPage) getRequestCycle().getPage(
				DuplicateSubmissionPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToTheSynchronizerToken() {
		TheSynchronizerTokenPage nextPage = (TheSynchronizerTokenPage) getRequestCycle().getPage(
				TheSynchronizerTokenPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToDoItOnceComponents() {
		DoItOnceComponentsPage nextPage = (DoItOnceComponentsPage) getRequestCycle().getPage(
				DoItOnceComponentsPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToDoItOnceAndRedirect() {
		DoItOnceAndRedirectPage nextPage = (DoItOnceAndRedirectPage) getRequestCycle().getPage(
				DoItOnceAndRedirectPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToDisablingValidators() {
		DisablingValidatorsPage nextPage = (DisablingValidatorsPage) getRequestCycle().getPage(
				DisablingValidatorsPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToWebFlow() {
		WebFlowStartPage nextPage = (WebFlowStartPage) getRequestCycle().getPage(WebFlowStartPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToWebFlow2() {
		WebFlow2StartPage nextPage = (WebFlow2StartPage) getRequestCycle().getPage(WebFlow2StartPage.PAGE_NAME);
		nextPage.activate();
	}

	public ILink doLogin() {
		ILink redirectTo = null;

		try {
			validate();

			if (hasErrors()) {
				return null;
			}

			// Authenticate the user, password

			ISecurityFinderSvcLocal finder = getBizServicesLocator().getSecurityFinderSvcLocal();
			User user = finder.authenticateUser(getLoginId(), getPassword());

			// Store the user and roles in the Visit

			Visit visit = getMyVisit();
			visit.noteLogIn(user);
			LOG.info(user.getLoginId() + " has logged in.");

			// Redirect to next page.
			// SECURITY NOTE - do not use simple page activation Instead, we
			// return an ILink because that instructs Tapestry to do
			// a redirect-after-post.
			// Redirect-after-post protects against a security vulnerability in
			// which a user may log in without a password simply by using a
			// browser that has been logged out, then using the Back button to
			// go back to the first screen after the login screen and
			// pressing refresh.
			// Without redirect-after-post this would re-post the login info
			// and they'd be logged in!

			redirectTo = getWelcomePageLink();
		}
		catch (ServiceException e) {
			recordError(e.toString());
		}
		catch (Exception e) {
			e.printStackTrace();
			recordError(getMessages().getMessage("login_problem"));
		}

		return redirectTo;
	}

	private void validate() {
		// None to do - validation has been specified in the HTML template
	}

	public void validate(IRequestCycle cycle) {
		super.validate(cycle);
	}

}
